@@ -362,7 +362,8 @@ class Setup(object):
             dlg.destroy()
             self.__flush_gtk_events()
         else:
-            message = _("IBus daemon coundn't be started in %d seconds")
+            # Translators: %d == 5 currently
+            message = _("IBus daemon could not be started in %d seconds")
             dlg = Gtk.MessageDialog(type = Gtk.MessageType.INFO,
                                     buttons = Gtk.ButtonsType.OK,
                                     text = message % timeout)
@@ -548,7 +549,12 @@ class Setup(object):
         Gtk.main()
 
 if __name__ == "__main__":
-    locale.setlocale(locale.LC_ALL, '')
+    try:
+        locale.setlocale(locale.LC_ALL, '')
+    except locale.Error:
+        print >> sys.stderr, "Using the fallback 'C' locale"
+        locale.setlocale(locale.LC_ALL, 'C')
+
     i18n_init()
     setup = Setup()
     setup.run()
