@@ -37,10 +37,13 @@ enum {
     PROP_AUTHOR,
     PROP_ICON,
     PROP_LAYOUT,
+    PROP_LAYOUT_VARIANT,
+    PROP_LAYOUT_OPTION,
     PROP_RANK,
     PROP_HOTKEYS,
     PROP_SYMBOL,
     PROP_SETUP,
+    PROP_VERSION,
 };
 
 
@@ -54,10 +57,13 @@ struct _IBusEngineDescPrivate {
     gchar      *author;
     gchar      *icon;
     gchar      *layout;
+    gchar      *layout_variant;
+    gchar      *layout_option;
     guint       rank;
     gchar      *hotkeys;
     gchar      *symbol;
     gchar      *setup;
+    gchar      *version;
 };
 
 #define IBUS_ENGINE_DESC_GET_PRIVATE(o)  \
@@ -209,6 +215,32 @@ ibus_engine_desc_class_init (IBusEngineDescClass *class)
                         "us",
                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+    /**
+     * IBusEngineDesc:layout-variant:
+     *
+     * The keyboard variant of engine description
+     */
+    g_object_class_install_property (gobject_class,
+                    PROP_LAYOUT_VARIANT,
+                    g_param_spec_string ("layout-variant",
+                        "description keyboard variant",
+                        "The keyboard variant of engine description",
+                        "",
+                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+    /**
+     * IBusEngineDesc:layout-option:
+     *
+     * The keyboard option of engine description
+     */
+    g_object_class_install_property (gobject_class,
+                    PROP_LAYOUT_OPTION,
+                    g_param_spec_string ("layout-option",
+                        "description keyboard option",
+                        "The keyboard option of engine description",
+                        "",
+                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
     /**
      * IBusEngineDesc:rank:
      *
@@ -262,6 +294,19 @@ ibus_engine_desc_class_init (IBusEngineDescClass *class)
                         "The exec lists of the engine setup command",
                         "",
                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+    /**
+     * IBusEngineDesc:version:
+     *
+     * The version number of engine description
+     */
+    g_object_class_install_property (gobject_class,
+                    PROP_VERSION,
+                    g_param_spec_string ("version",
+                        "version number",
+                        "The version number of engine description",
+                        "",
+                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 static void
@@ -277,10 +322,13 @@ ibus_engine_desc_init (IBusEngineDesc *desc)
     desc->priv->author = NULL;
     desc->priv->icon = NULL;
     desc->priv->layout = NULL;
+    desc->priv->layout_variant = NULL;
+    desc->priv->layout_option = NULL;
     desc->priv->rank = 0;
     desc->priv->hotkeys = NULL;
     desc->priv->symbol = NULL;
     desc->priv->setup = NULL;
+    desc->priv->version = NULL;
 }
 
 static void
@@ -294,9 +342,12 @@ ibus_engine_desc_destroy (IBusEngineDesc *desc)
     g_free (desc->priv->author);
     g_free (desc->priv->icon);
     g_free (desc->priv->layout);
+    g_free (desc->priv->layout_variant);
+    g_free (desc->priv->layout_option);
     g_free (desc->priv->hotkeys);
     g_free (desc->priv->symbol);
     g_free (desc->priv->setup);
+    g_free (desc->priv->version);
 
     IBUS_OBJECT_CLASS (ibus_engine_desc_parent_class)->destroy (IBUS_OBJECT (desc));
 }
@@ -340,6 +391,14 @@ ibus_engine_desc_set_property (IBusEngineDesc *desc,
         g_assert (desc->priv->layout == NULL);
         desc->priv->layout = g_value_dup_string (value);
         break;
+    case PROP_LAYOUT_VARIANT:
+        g_assert (desc->priv->layout_variant == NULL);
+        desc->priv->layout_variant = g_value_dup_string (value);
+        break;
+    case PROP_LAYOUT_OPTION:
+        g_assert (desc->priv->layout_option == NULL);
+        desc->priv->layout_option = g_value_dup_string (value);
+        break;
     case PROP_RANK:
         desc->priv->rank = g_value_get_uint (value);
         break;
@@ -355,6 +414,10 @@ ibus_engine_desc_set_property (IBusEngineDesc *desc,
         g_assert (desc->priv->setup == NULL);
         desc->priv->setup = g_value_dup_string (value);
         break;
+    case PROP_VERSION:
+        g_assert (desc->priv->version == NULL);
+        desc->priv->version = g_value_dup_string (value);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (desc, prop_id, pspec);
     }
@@ -391,6 +454,12 @@ ibus_engine_desc_get_property (IBusEngineDesc *desc,
     case PROP_LAYOUT:
         g_value_set_string (value, ibus_engine_desc_get_layout (desc));
         break;
+    case PROP_LAYOUT_VARIANT:
+        g_value_set_string (value, ibus_engine_desc_get_layout_variant (desc));
+        break;
+    case PROP_LAYOUT_OPTION:
+        g_value_set_string (value, ibus_engine_desc_get_layout_option (desc));
+        break;
     case PROP_RANK:
         g_value_set_uint (value, ibus_engine_desc_get_rank (desc));
         break;
@@ -403,6 +472,9 @@ ibus_engine_desc_get_property (IBusEngineDesc *desc,
     case PROP_SETUP:
         g_value_set_string (value, ibus_engine_desc_get_setup (desc));
         break;
+    case PROP_VERSION:
+        g_value_set_string (value, ibus_engine_desc_get_version (desc));
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (desc, prop_id, pspec);
     }
@@ -432,9 +504,13 @@ ibus_engine_desc_serialize (IBusEngineDesc  *desc,
     g_variant_builder_add (builder, "s", NOTNULL (desc->priv->icon));
     g_variant_builder_add (builder, "s", NOTNULL (desc->priv->layout));
     g_variant_builder_add (builder, "u", desc->priv->rank);
+    /* The serialized order should be kept. */
     g_variant_builder_add (builder, "s", NOTNULL (desc->priv->hotkeys));
     g_variant_builder_add (builder, "s", NOTNULL (desc->priv->symbol));
     g_variant_builder_add (builder, "s", NOTNULL (desc->priv->setup));
+    g_variant_builder_add (builder, "s", NOTNULL (desc->priv->layout_variant));
+    g_variant_builder_add (builder, "s", NOTNULL (desc->priv->layout_option));
+    g_variant_builder_add (builder, "s", NOTNULL (desc->priv->version));
 #undef NOTNULL
 
     return TRUE;
@@ -462,9 +538,17 @@ ibus_engine_desc_deserialize (IBusEngineDesc *desc,
     g_variant_get_child (variant, retval++, "s", &desc->priv->icon);
     g_variant_get_child (variant, retval++, "s", &desc->priv->layout);
     g_variant_get_child (variant, retval++, "u", &desc->priv->rank);
+    /* The serialized order should be kept. */
     g_variant_get_child (variant, retval++, "s", &desc->priv->hotkeys);
     g_variant_get_child (variant, retval++, "s", &desc->priv->symbol);
     g_variant_get_child (variant, retval++, "s", &desc->priv->setup);
+    if (g_variant_n_children (variant) < retval + 2)
+        return retval;
+    g_variant_get_child (variant, retval++, "s", &desc->priv->layout_variant);
+    g_variant_get_child (variant, retval++, "s", &desc->priv->layout_option);
+    if (g_variant_n_children (variant) < retval + 1)
+        return retval;
+    g_variant_get_child (variant, retval++, "s", &desc->priv->version);
 
     return retval;
 }
@@ -487,10 +571,13 @@ ibus_engine_desc_copy (IBusEngineDesc       *dest,
     dest->priv->author           = g_strdup (src->priv->author);
     dest->priv->icon             = g_strdup (src->priv->icon);
     dest->priv->layout           = g_strdup (src->priv->layout);
+    dest->priv->layout_variant   = g_strdup (src->priv->layout_variant);
+    dest->priv->layout_option    = g_strdup (src->priv->layout_option);
     dest->priv->rank             = src->priv->rank;
     dest->priv->hotkeys          = g_strdup (src->priv->hotkeys);
     dest->priv->symbol           = g_strdup (src->priv->symbol);
     dest->priv->setup            = g_strdup (src->priv->setup);
+    dest->priv->version          = g_strdup (src->priv->version);
     return TRUE;
 }
 
@@ -527,9 +614,12 @@ ibus_engine_desc_output (IBusEngineDesc *desc,
     OUTPUT_ENTRY_1(author);
     OUTPUT_ENTRY_1(icon);
     OUTPUT_ENTRY_1(layout);
+    OUTPUT_ENTRY_1(layout_variant);
+    OUTPUT_ENTRY_1(layout_option);
     OUTPUT_ENTRY_1(hotkeys);
     OUTPUT_ENTRY_1(symbol);
     OUTPUT_ENTRY_1(setup);
+    OUTPUT_ENTRY_1(version);
     g_string_append_indent (output, indent + 1);
     g_string_append_printf (output, "<rank>%u</rank>\n", desc->priv->rank);
 #undef OUTPUT_ENTRY
@@ -562,9 +652,12 @@ ibus_engine_desc_parse_xml_node (IBusEngineDesc *desc,
         PARSE_ENTRY_1(author);
         PARSE_ENTRY_1(icon);
         PARSE_ENTRY_1(layout);
+        PARSE_ENTRY_1(layout_variant);
+        PARSE_ENTRY_1(layout_option);
         PARSE_ENTRY_1(hotkeys);
         PARSE_ENTRY_1(symbol);
         PARSE_ENTRY_1(setup);
+        PARSE_ENTRY_1(version);
 #undef PARSE_ENTRY
 #undef PARSE_ENTRY_1
         if (g_strcmp0 (sub_node->name , "rank") == 0) {
@@ -591,10 +684,13 @@ IBUS_ENGINE_DESC_GET_PROPERTY (license, const gchar *)
 IBUS_ENGINE_DESC_GET_PROPERTY (author, const gchar *)
 IBUS_ENGINE_DESC_GET_PROPERTY (icon, const gchar *)
 IBUS_ENGINE_DESC_GET_PROPERTY (layout, const gchar *)
+IBUS_ENGINE_DESC_GET_PROPERTY (layout_variant, const gchar *)
+IBUS_ENGINE_DESC_GET_PROPERTY (layout_option, const gchar *)
 IBUS_ENGINE_DESC_GET_PROPERTY (rank, guint)
 IBUS_ENGINE_DESC_GET_PROPERTY (hotkeys, const gchar *)
 IBUS_ENGINE_DESC_GET_PROPERTY (symbol, const gchar *)
 IBUS_ENGINE_DESC_GET_PROPERTY (setup, const gchar *)
+IBUS_ENGINE_DESC_GET_PROPERTY (version, const gchar *)
 #undef IBUS_ENGINE_DESC_GET_PROPERTY
 
 IBusEngineDesc *
@@ -641,9 +737,12 @@ ibus_engine_desc_new_varargs (const gchar *first_property_name, ...)
     g_assert (desc->priv->author);
     g_assert (desc->priv->icon);
     g_assert (desc->priv->layout);
+    g_assert (desc->priv->layout_variant);
+    g_assert (desc->priv->layout_option);
     g_assert (desc->priv->hotkeys);
     g_assert (desc->priv->symbol);
     g_assert (desc->priv->setup);
+    g_assert (desc->priv->version);
 
     return desc;
 }
